var path = require('path');
var fs = require('fs');

var BrowserSync = require('browser-sync');

/**
 * Starts the browser sync server, and when files are changed, does the reload
 * @param {Object} opts - Options Object to be passed to browserSync. If this is a function, the function is called with default values and should return the final options to be passed to browser-sync
 * @param {Function} cb - A callback when server is ready, calls with (err, servr_hostname)
 */
module.exports = function(opts, cb) {
    opts = opts || {};
    var bs = BrowserSync.create();

    var defaults = {
        logFileChanges: true,
        logConnections: true,
        open: false,
        snippetOptions: {
            rule: {
                match: /<\/body>/i,
                fn: function(snippet, match) {
                    return snippet + match;
                }
            }
        },
        minify: false,
        watchOptions: {},
        files: [],
        cors: true,
        https: false
    };

    if (typeof opts === 'function') {
        opts = opts(defaults);
    } else {
        for (var key in defaults) {
            if (typeof opts[key] === 'undefined') {
                opts[key] = defaults[key];
            }
        }
    }

    bs.init(opts, function(err, bs) {
        var urls = bs.options.getIn(['urls']);
        var servers = {};
        ['local', 'external', 'tunnel'].forEach(function(type) {
            servers[type] = urls.get(type);
        });
        cb(err, servers);
    });
    return bs;
};
